bl_info = {
    "name": "Material Simplifier",
    "author": "Free3D.xyz",
    "version": (1, 2),
    "blender": (4, 2, 0),
    "location": "Properties > Material",
    "description": "Renders selected materials' principled shader inputs to new images and sets them up in shader graph for easy glTF export",
    "warning": "",
    "doc_url": "",
    "category": "Material",
}

import bpy
import os
from mathutils import Vector

class MaterialRendererPanel(bpy.types.Panel):
    bl_label = "Material Simplifier"
    bl_idname = "OBJECT_PT_material_renderer"
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = "material"

    def draw(self, context):
        layout = self.layout
        scene = context.scene
        layout.prop_search(scene, "selected_material", bpy.data, "materials", text="Material")
        layout.operator("object.render_material")

def check_srgb_basecolor(input_socket, is_emission=False):
    while True:
        if not input_socket.is_linked:
            return False
        linked_node = input_socket.links[0].from_node
        if linked_node.type == 'TEX_IMAGE':
            return linked_node.image and linked_node.image.colorspace_settings.name != "Non-Color"
        elif linked_node.type == 'RGB':
            return False
        elif is_emission and "MIX" in linked_node.type.upper():
            return True
        color_input = linked_node.inputs.get("Color") or linked_node.inputs.get("Color1") or linked_node.inputs.get("A")
        if not color_input or not color_input.links:
            return False
        input_socket = color_input.links[0].from_socket

class RenderMaterialOperator(bpy.types.Operator):
    bl_idname = "object.render_material"
    bl_label = "Render Material"

    def execute(self, context):
        selected_material = context.scene.selected_material
        if not selected_material:
            self.report({'WARNING'}, "No material selected")
            return {'FINISHED'}
        
        self.report({'INFO'}, f"Processing material: {selected_material.name}")
        
        principled_bsdf = next((node for node in selected_material.node_tree.nodes if node.type == 'BSDF_PRINCIPLED'), None)
        self.use_srgb_base_color = check_srgb_basecolor(principled_bsdf.inputs['Base Color']) if principled_bsdf else False
        self.use_srgb_emission = check_srgb_basecolor(principled_bsdf.inputs['Emission Color'], is_emission=True) if principled_bsdf else False

        self.set_texture_properties(selected_material)

        max_resolution = self.get_max_resolution(selected_material)
        plane = self.create_plane(context, max_resolution)
        plane.data.materials.append(selected_material)
        camera = self.setup_scene_and_camera(context, plane)

        self.rendered_images = {}
        self.process_inputs(context, selected_material, max_resolution, plane)

        self.setup_new_material(selected_material)

        bpy.data.objects.remove(plane, do_unlink=True)
        bpy.data.objects.remove(camera, do_unlink=True)

        self.report({'INFO'}, "All renders completed and material set up")
        self.sort_nodes(selected_material)
                    
        return {'FINISHED'}

    def get_max_resolution(self, material):
        max_res = (0, 0)
        for node in material.node_tree.nodes:
            if node.type == 'TEX_IMAGE' and node.image:
                max_res = max(max_res, node.image.size, key=lambda x: x[0] * x[1])
        return max_res

    def set_texture_properties(self, material):
        for node in material.node_tree.nodes:
            if node.type == 'TEX_IMAGE':
                node.interpolation = 'Closest'
                node.image.colorspace_settings.name = 'Non-Color'

    def create_plane(self, context, resolution):
        bpy.ops.mesh.primitive_plane_add()
        plane = context.active_object
        scale_x = resolution[0] / max(resolution)
        scale_y = resolution[1] / max(resolution)
        plane.scale = (scale_x, scale_y, 1)
        return plane

    def setup_scene_and_camera(self, context, obj):
        cam_data = bpy.data.cameras.new(name='Material_Render_Camera')
        cam = bpy.data.objects.new('Material_Render_Camera', cam_data)
        context.scene.collection.objects.link(cam)
        context.scene.camera = cam

        cam.data.type = 'ORTHO'
        cam.data.clip_start = 0.01
        cam.data.clip_end = 1000
        cam.rotation_euler = (0, 0, 0)
        cam.location = (0, 0, 10)
        cam.data.ortho_scale = max(obj.scale[0], obj.scale[1]) * 2

        context.scene.render.engine = 'BLENDER_EEVEE_NEXT'

        return cam

    def process_inputs(self, context, material, resolution, plane):
        principled_bsdf = next((node for node in material.node_tree.nodes if node.type == 'BSDF_PRINCIPLED'), None)
        gltf_output = next((node for node in material.node_tree.nodes if node.type == 'GROUP' and node.node_tree.name == 'glTF Material Output'), None)
        material_output = next((node for node in material.node_tree.nodes if node.type == 'OUTPUT_MATERIAL'), None)

        inputs_to_render = [
            ('Base Color', principled_bsdf.inputs['Base Color'] if principled_bsdf else None),
            ('Metallic', principled_bsdf.inputs['Metallic'] if principled_bsdf else None),
            ('Roughness', principled_bsdf.inputs['Roughness'] if principled_bsdf else None),
            ('Alpha', principled_bsdf.inputs['Alpha'] if principled_bsdf else None),
            ('Normal', principled_bsdf.inputs['Normal'] if principled_bsdf else None),
            ('Emission Color', principled_bsdf.inputs['Emission Color'] if principled_bsdf else None),
            ('Occlusion', gltf_output.inputs['Occlusion'] if gltf_output else None)
        ]

        original_display_device = context.scene.display_settings.display_device
        original_view_transform = context.scene.view_settings.view_transform
        original_look = context.scene.view_settings.look

        for input_name, input_socket in inputs_to_render:
            print(input_name, input_socket)
            if input_socket and input_socket.is_linked:
                rendered_image = self.render_input(context, material, input_name, input_socket, material_output, resolution, plane)
                if rendered_image:
                    self.rendered_images[input_name] = rendered_image

        context.scene.display_settings.display_device = original_display_device
        context.scene.view_settings.view_transform = original_view_transform
        context.scene.view_settings.look = original_look

    def render_input(self, context, material, input_name, input_socket, material_output, resolution, plane):
        original_link = input_socket.links[0]
        
        if input_name == 'Normal':
            normal_map_node = original_link.from_node
            if normal_map_node.type == 'NORMAL_MAP':
                color_input = normal_map_node.inputs['Color']
                if color_input.is_linked:
                    material.node_tree.links.new(color_input.links[0].from_socket, material_output.inputs['Surface'])
                else:
                    return None
            else:
                return None
        elif input_name == 'Occlusion':
            material.node_tree.links.new(original_link.from_socket, material_output.inputs['Surface'])
        elif input_name == 'Alpha':
            is_clip, round_node = self.check_if_clip(material)
            if is_clip:
                material.node_tree.links.new(round_node.inputs[0].links[0].from_socket, material_output.inputs['Surface'])
            else:
                material.node_tree.links.new(original_link.from_socket, material_output.inputs['Surface'])
        else:
            material.node_tree.links.new(original_link.from_socket, material_output.inputs['Surface'])
        
        context.scene.render.resolution_x = resolution[0]
        context.scene.render.resolution_y = resolution[1]
        
        context.scene.display_settings.display_device = 'sRGB'
        
        if (input_name == 'Base Color' and not self.use_srgb_base_color) or \
           (input_name == 'Emission Color' and not self.use_srgb_emission):
            context.scene.view_settings.view_transform = 'Standard'
        else:
            context.scene.view_settings.view_transform = 'Raw'
        
        context.scene.view_settings.look = 'None'
        
        for obj in bpy.data.objects:
            obj.hide_render = obj != plane
        
        material_name = self.clean_material_name(material.name)
        output_name = f"T_{material_name}_{input_name.lower().replace(' ', '_')}"
        
        textures_dir = bpy.path.abspath("//textures")
        os.makedirs(textures_dir, exist_ok=True)
        
        context.scene.render.filepath = os.path.join(textures_dir, output_name)
        
        bpy.ops.render.render(write_still=True)
        
        image = bpy.data.images.load(context.scene.render.filepath + ".png")
        image.name = output_name
        image.use_fake_user = True
        
        self.report({'INFO'}, f"Rendered {output_name}")
        
        for obj in bpy.data.objects:
            obj.hide_render = False
        
        material.node_tree.links.new(original_link.from_socket, original_link.to_socket)
        
        return image

    def clean_material_name(self, name):
        prefixes = ['m_', 'mi_', 'mt_', 'mat_', 'mtr_', 'material_', 'mr_']
        suffixes = ['_m', '_mi', '_mt', '_mat', '_mtr', '_material', '_mr']
        
        name_lower = name.lower()
        for prefix in prefixes:
            if name_lower.startswith(prefix):
                name = name[len(prefix):]
                break
        
        for suffix in suffixes:
            if name_lower.endswith(suffix):
                name = name[:-len(suffix)]
                break
        
        return name

    def check_if_clip(self, material):
        principled_bsdf = next((node for node in material.node_tree.nodes if node.type == 'BSDF_PRINCIPLED'), None)
        if principled_bsdf and principled_bsdf.inputs['Alpha'].is_linked:
            linked_node = principled_bsdf.inputs['Alpha'].links[0].from_node
            if linked_node.type == 'MATH' and linked_node.operation == 'ROUND':
                return True, linked_node
        return False, None

    def setup_new_material(self, material):
        nodes_to_keep = ['Principled BSDF', 'Material Output']
        gltf_output = next((node for node in material.node_tree.nodes if node.type == 'GROUP' and node.node_tree.name == 'glTF Material Output'), None)
        if gltf_output:
            nodes_to_keep.append(gltf_output.name)

        is_clip, round_node = self.check_if_clip(material)
        if is_clip:
            nodes_to_keep.append(round_node.name)

        for node in material.node_tree.nodes:
            if node.name not in nodes_to_keep:
                material.node_tree.nodes.remove(node)

        principled_bsdf = material.node_tree.nodes.get('Principled BSDF')
        material_output = material.node_tree.nodes.get('Material Output')

        for input_name, rendered_image in self.rendered_images.items():
            tex_node = material.node_tree.nodes.new('ShaderNodeTexImage')
            tex_node.image = rendered_image
            tex_node.name = f"{input_name} Texture"
            
            if input_name in ['Base Color', 'Emission Color']:
                tex_node.image.colorspace_settings.name = 'sRGB'
            else:
                tex_node.image.colorspace_settings.name = 'Non-Color'

            if input_name == 'Normal':
                normal_map_node = material.node_tree.nodes.new('ShaderNodeNormalMap')
                material.node_tree.links.new(tex_node.outputs['Color'], normal_map_node.inputs['Color'])
                material.node_tree.links.new(normal_map_node.outputs['Normal'], principled_bsdf.inputs['Normal'])
            elif input_name == 'Occlusion' and gltf_output:
                material.node_tree.links.new(tex_node.outputs['Color'], gltf_output.inputs['Occlusion'])
            elif input_name == 'Alpha' and is_clip:
                material.node_tree.links.new(tex_node.outputs['Color'], round_node.inputs[0])
                material.node_tree.links.new(round_node.outputs[0], principled_bsdf.inputs['Alpha'])
            elif input_name == 'Emission Color':
                material.node_tree.links.new(tex_node.outputs['Color'], principled_bsdf.inputs[input_name])
                principled_bsdf.inputs['Emission Strength'].default_value = 1.0
            else:
                material.node_tree.links.new(tex_node.outputs['Color'], principled_bsdf.inputs[input_name])

        material.node_tree.links.new(principled_bsdf.outputs['BSDF'], material_output.inputs['Surface'])
    
    def sort_nodes(self, material):
        nodes = material.node_tree.nodes
        ignored_nodes = ["BSDF_PRINCIPLED", "OUTPUT_MATERIAL", "YIGLTF_MATERIAL_OUTPUT"]
        nodes_to_move = [node for node in nodes if node.type not in ignored_nodes]
        for i, node in enumerate(nodes_to_move):
            node.hide = True
            node.location = Vector((-300, 200 - i * 40))

def register():
    bpy.utils.register_class(MaterialRendererPanel)
    bpy.utils.register_class(RenderMaterialOperator)
    bpy.types.Scene.selected_material = bpy.props.PointerProperty(type=bpy.types.Material)

def unregister():
    bpy.utils.unregister_class(MaterialRendererPanel)
    bpy.utils.unregister_class(RenderMaterialOperator)
    del bpy.types.Scene.selected_material

if __name__ == "__main__":
    register()
